@@ -7,9 +7,9 @@ from metaapi_cloud_sdk.clients.metaApi.tradeException import TradeException
 
 token = os.getenv('TOKEN') or '<put in your token here>'
 accountId = os.getenv('ACCOUNT_ID') or '<put in your account id here>'
-api = MetaApi(token)
 
 async def test_meta_api_synchronization():
+    api = MetaApi(token)
     try:
         account = await api.metatrader_account_api.get_account(accountId)
         initial_state = account.state
@@ -48,15 +48,16 @@ async def test_meta_api_synchronization():
             result = await connection.create_limit_buy_order('GBPUSD', 0.07, 1.0, 0.9, 2.0,
                                                              {'comment': 'comm', 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
             print('Trade successful, result code is ' + result['stringCode'])
-        except TradeException as err:
-            print('Trade failed with result code ' + err.stringCode)
+        except Exception as err:
+            print('Trade failed with error:')
+            print(api.format_error(err))
 
-        if initialState not in deployedStates:
+        if initial_state not in deployed_states:
             # undeploy account if it was undeployed
             print('Undeploying account')
             await account.undeploy()
 
     except Exception as err:
-        print(err)
+        print(api.format_error(err))
 
 asyncio.run(test_meta_api_synchronization())
