@@ -11,9 +11,10 @@ login = os.getenv('LOGIN') or '<put in your MT login here>'
 password = os.getenv('PASSWORD') or '<put in your MT password here>'
 server_name = os.getenv('SERVER') or '<put in your MT server name here>'
 server_dat_file = os.getenv('PATH_TO_SERVERS_DAT') or '/path/to/your/servers.dat'
-api = MetaApi(token)
+
 
 async def test_meta_api_synchronization():
+    api = MetaApi(token)
     try:
         profiles = await api.provisioning_profile_api.get_provisioning_profiles()
 
@@ -95,14 +96,16 @@ async def test_meta_api_synchronization():
             result = await connection.create_limit_buy_order('GBPUSD', 0.07, 1.0, 0.9, 2.0,
                                                              {'comment': 'comm', 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
             print('Trade successful, result code is ' + result['stringCode'])
-        except TradeException as err:
-            print('Trade failed with result code ' + err.stringCode)
+        except Exception as err:
+            print('Trade failed with error:')
+            print(api.format_error(err))
 
         # finally, undeploy account after the test
         print('Undeploying MT5 account so that it does not consume any unwanted resources')
         await account.undeploy()
+
     except Exception as err:
-        print(err)
+        print(api.format_error(err))
 
 asyncio.run(test_meta_api_synchronization())
 
