@@ -3,13 +3,36 @@ from typing import List, Optional
 from datetime import datetime
 
 
+class CopyFactoryStrategyIdAndName(TypedDict):
+    """CopyFactory strategy id and name."""
+    id: str
+    """Unique strategy id."""
+    name: str
+    """Human-readable strategy name."""
+
+
+class CopyFactoryStrategyStopout(TypedDict):
+    """CopyFactory strategy stopout."""
+    strategy: CopyFactoryStrategyIdAndName
+    """Strategy which was stopped out."""
+    reason: str
+    """Stopout reason. One of yearly-balance, monthly-balance, daily-balance, yearly-equity, monthly-equity,
+    daily-equity, max-drawdown"""
+    reasonDescription: str
+    """Human-readable description of the stopout reason."""
+    stoppedAt: datetime
+    """Time the strategy was stopped at."""
+    stoppedTill: datetime
+    """Time the strategy is stopped till."""
+
+
 class StrategyId(TypedDict):
     """Strategy id"""
     id: str
     """Strategy id"""
 
 
-class CopyFactoryStrategyStopOut(TypedDict):
+class CopyFactoryStrategyStopOutSettings(TypedDict):
     """CopyFactory strategy stopout settings."""
     value: float
     """Value of the stop out risk, expressed as a fraction of 1."""
@@ -103,11 +126,13 @@ class CopyFactoryStrategySubscription(TypedDict):
     maxTradeRisk: Optional[float]
     """Optional max risk per trade, expressed as a fraction of 1. If trade has a SL, the trade size will be adjusted to
     match the risk limit. If not, the trade SL will be applied according to the risk limit."""
+    reverse: bool
+    """Flag indicating that the strategy should be copied in a reverse direction."""
     reduceCorrelations: Optional[str]
     """Optional setting indicating whether to enable automatic trade correlation reduction. Possible settings are not
     specified (disable correlation risk restrictions), by-strategy (limit correlations on strategy level) or by-symbol
     (limit correlations on symbol level)."""
-    stopOutRisk: Optional[CopyFactoryStrategyStopOut]
+    stopOutRisk: Optional[CopyFactoryStrategyStopOutSettings]
     """Optional stop out setting. All trading will be terminated and positions closed once equity drawdown reaches
     this value."""
     symbolFilter: Optional[CopyFactoryStrategySymbolFilter]
@@ -145,6 +170,12 @@ class CopyFactoryAccountUpdate(TypedDict):
     allowed to open new positions with a symbol equal to the symbol of an existing strategy position (can be used to
     gracefully exit strategies trading in netting mode or placing a series of related trades per symbol). One of
     by-position, by-symbol or leave empty to disable this setting."""
+    stopOutRisk: Optional[CopyFactoryStrategyStopout]
+    """Optional stop out setting. All trading will be terminated and positions closed once equity drawdown reaches
+    this value."""
+    riskLimits: Optional[List[CopyFactoryStrategyRiskLimit]]
+    """Optional account risk limits. You can configure trading to be stopped once total drawdown generated during
+    specific period is exceeded. Can be specified either for balance or equity drawdown."""
     maxLeverage: Optional[float]
     """Optional setting indicating maxumum leverage allowed when opening a new positions. Any trade which results in a
     higher leverage will be discarded."""
@@ -217,11 +248,13 @@ class CopyFactoryStrategyUpdate(TypedDict):
     maxTradeRisk: Optional[float]
     """Optional max risk per trade, expressed as a fraction of 1. If trade has a SL, the trade size will be adjusted to
     match the risk limit. If not, the trade SL will be applied according to the risk limit."""
+    reverse: bool
+    """Flag indicating that the strategy should be copied in a reverse direction."""
     reduceCorrelations: Optional[str]
     """Optional setting indicating whether to enable automatic trade correlation reduction. Possible settings are not
     specified (disable correlation risk restrictions), by-strategy (limit correlations on strategy level) or by-symbol
     (limit correlations on symbol level)."""
-    stopOutRisk: Optional[CopyFactoryStrategyStopOut]
+    stopOutRisk: Optional[CopyFactoryStrategyStopOutSettings]
     """Optional stop out setting. All trading will be terminated and positions closed once equity drawdown reaches
     this value."""
     symbolFilter: Optional[CopyFactoryStrategySymbolFilter]
@@ -243,14 +276,6 @@ class CopyFactoryStrategyUpdate(TypedDict):
     being opened at source and let the position to live for up to 90 days."""
 
 
-class CopyFactoryStrategyIdAndName(TypedDict):
-    """CopyFactory strategy id and name."""
-    id: str
-    """Unique strategy id."""
-    name: str
-    """Human-readable strategy name."""
-
-
 class CopyFactorySubscriberOrProvider(TypedDict):
     """CopyFactory provider or subscriber"""
     id: str
@@ -313,8 +338,10 @@ class CopyFactoryTransaction(TypedDict):
     """Provider commission."""
     platformCommission: float
     """Platform commission."""
-    quantity: Optional[float]
-    """Trade volume."""
+    incomingProviderCommission: Optional[float]
+    """Commission paid by provider to underlying providers."""
+    incomingPlatformCommission: Optional[float]
+    """Platform commission paid by provider to underlying providers."""
     lotPrice: Optional[float]
     """Trade lot price."""
     tickPrice: Optional[float]
@@ -331,21 +358,6 @@ class CopyFactoryTransaction(TypedDict):
     """trade copying metrics such as slippage and latencies. Measured selectively for copied trades"""
 
 
-class CopyFactoryStrategyStopout(TypedDict):
-    """CopyFactory strategy stopout."""
-    strategy: CopyFactoryStrategyIdAndName
-    """Strategy which was stopped out."""
-    reason: str
-    """Stopout reason. One of yearly-balance, monthly-balance, daily-balance, yearly-equity, monthly-equity,
-    daily-equity, max-drawdown"""
-    reasonDescription: str
-    """Human-readable description of the stopout reason."""
-    stoppedAt: datetime
-    """Time the strategy was stopped at."""
-    stoppedTill: datetime
-    """Time the strategy is stopped till."""
-
-
 class ResynchronizationTask(TypedDict):
     """Resynchronization task."""
     _id: str
@@ -356,3 +368,85 @@ class ResynchronizationTask(TypedDict):
     """The time task was created at."""
     status: str
     """Task status. One of PLANNED, EXECUTING, SYNCHRONIZING."""
+
+
+class CopyFactoryPortfolioMember(TypedDict):
+    """Portfolio strategy member."""
+    strategyId: str
+    """Member strategy id."""
+    multiplier: float
+    """Copying multiplier (weight in the portfolio)."""
+    skipPendingOrders: Optional[bool]
+    """Optional flag indicating that pending orders should not be copied. Default is to copy pending orders."""
+    maxTradeRisk: Optional[float]
+    """Optional max risk per trade, expressed as a fraction of 1. If trade has a SL, the trade size will be adjusted
+    to match the risk limit. If not, the trade SL will be applied according to the risk limit."""
+    reverse: bool
+    """Flag indicating that the strategy should be copied in a reverse direction."""
+    reduceCorrelations: Optional[str]
+    """Optional setting indicating whether to enable automatic trade correlation reduction. Possible settings are
+    not specified (disable correlation risk restrictions), by-strategy (limit correlations on strategy level) or
+    by-symbol (limit correlations on symbol level)."""
+    stopOutRisk: Optional[CopyFactoryStrategyStopOutSettings]
+    """Optional stop out setting. All trading will be terminated and positions closed once equity drawdown reaches this
+    value."""
+    symbolFilter: Optional[CopyFactoryStrategySymbolFilter]
+    """Symbol filters which can be used to copy only specific symbols or exclude some symbols from copying."""
+    newsFilter: Optional[CopyFactoryStrategyNewsFilter]
+    """News risk filter configuration."""
+    riskLimits: Optional[List[CopyFactoryStrategyRiskLimit]]
+    """Optional strategy risk limits. You can configure trading to be stopped once total drawdown generated during
+    specific period is exceeded. Can be specified either for balance or equity drawdown."""
+    maxStopLoss: Optional[CopyFactoryStrategyMaxStopLoss]
+    """Optional stop loss value restriction."""
+    maxLeverage: Optional[float]
+    """Optional max leverage risk restriction. All trades resulting in a leverage value higher than specified will be
+    skipped."""
+
+
+class CopyFactoryPortfolioStrategyUpdate(TypedDict):
+    """Portfolio strategy update."""
+    name: str
+    """Strategy human-readable name."""
+    description: str
+    """Longer strategy human-readable description."""
+    members: List[CopyFactoryPortfolioMember]
+    """Array of portfolio memebers."""
+    commissionScheme: Optional[CopyFactoryStrategyCommissionScheme]
+    """Commission scheme allowed by this strategy. By default monthly billing period with no commission is being
+    used."""
+    skipPendingOrders: Optional[bool]
+    """Optional flag indicating that pending orders should not be copied. Default is to copy pending orders."""
+    maxTradeRisk: Optional[float]
+    """Optional max risk per trade, expressed as a fraction of 1. If trade has a SL, the trade size will be adjusted
+    to match the risk limit. If not, the trade SL will be applied according to the risk limit."""
+    reverse: bool
+    """Flag indicating that the strategy should be copied in a reverse direction."""
+    reduceCorrelations: Optional[str]
+    """Optional setting indicating whether to enable automatic trade correlation reduction. Possible settings are not
+    specified (disable correlation risk restrictions), by-strategy (limit correlations on strategy level) or by-symbol
+    (limit correlations on symbol level)."""
+    stopOutRisk: Optional[CopyFactoryStrategyStopOutSettings]
+    """Optional stop out setting. All trading will be terminated and positions closed once equity drawdown reaches this
+    value."""
+    symbolFilter: Optional[CopyFactoryStrategySymbolFilter]
+    """Symbol filters which can be used to copy only specific symbols or exclude some symbols from copying."""
+    newsFilter: Optional[CopyFactoryStrategyNewsFilter]
+    """News risk filter configuration."""
+    riskLimits: Optional[List[CopyFactoryStrategyRiskLimit]]
+    """Optional strategy risk limits. You can configure trading to be stopped once total drawdown generated during
+    specific period is exceeded. Can be specified either for balance or equity drawdown."""
+    maxStopLoss: Optional[CopyFactoryStrategyMaxStopLoss]
+    """Optional stop loss value restriction."""
+    maxLeverage: Optional[float]
+    """Optional max leverage risk restriction. All trades resulting in a leverage value higher than specified will be
+    skipped."""
+
+
+class CopyFactoryPortfolioStrategy(CopyFactoryPortfolioStrategyUpdate):
+    """Portfolio strategy, i.e. the strategy which includes a set of other strategies."""
+    _id: str
+    """Unique strategy id."""
+    platformCommissionRate: float
+    """Commission rate the platform charges for strategy copying, applied to commissions charged by provider. This
+    commission applies only to accounts not managed directly by provider. Should be fraction of 1."""
