@@ -2,6 +2,7 @@ from ..clients.metaApi.synchronizationListener import SynchronizationListener
 from ..clients.metaApi.reconnectListener import ReconnectListener
 from ..clients.metaApi.metaApiWebsocket_client import MetaApiWebsocketClient
 from .terminalState import TerminalState
+from .connectionHealthMonitor import ConnectionHealthMonitor
 from .memoryHistoryStorage import MemoryHistoryStorage
 from .metatraderAccountModel import MetatraderAccountModel
 from .connectionRegistryModel import ConnectionRegistryModel
@@ -11,9 +12,11 @@ from .models import random_id, MetatraderSymbolSpecification, MetatraderAccountI
     MetatraderPosition, MetatraderOrder, MetatraderHistoryOrders, MetatraderDeals, MetatraderTradeResponse, \
     MetatraderSymbolPrice, MarketTradeOptions, PendingTradeOptions
 from datetime import datetime, timedelta
-from typing import Coroutine, List, TypedDict, Optional
+from typing import Coroutine, List, Optional, Dict
+from typing_extensions import TypedDict
 import pytz
 import asyncio
+from threading import Timer
 
 
 class SynchronizationOptions(TypedDict):
@@ -45,7 +48,6 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         super().__init__()
         self._websocketClient = websocket_client
         self._account = account
-        self._synchronized = False
         self._closed = False
         self._ordersSynchronized = {}
         self._dealsSynchronized = {}
@@ -55,10 +57,15 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         self._history_start_time = history_start_time
         self._terminalState = TerminalState()
         self._historyStorage = history_storage or MemoryHistoryStorage(account.id)
+        self._healthMonitor = ConnectionHealthMonitor(self)
         self._websocketClient.add_synchronization_listener(account.id, self)
         self._websocketClient.add_synchronization_listener(account.id, self._terminalState)
         self._websocketClient.add_synchronization_listener(account.id, self._historyStorage)
+        self._websocketClient.add_synchronization_listener(account.id, self._healthMonitor)
         self._websocketClient.add_reconnect_listener(self)
+        self._subscriptions = {}
+        self._synchronized = False
+        self._shouldSynchronize = None
 
     def get_account_information(self) -> 'Coroutine[asyncio.Future[MetatraderAccountInformation]]':
         """Returns account information (see
@@ -192,15 +199,18 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_deals_by_time_range(self._account.id, start_time, end_time, offset, limit)
 
-    def remove_history(self) -> Coroutine:
+    def remove_history(self, application: str = None) -> Coroutine:
         """Clears the order and transaction history of a specified account so that it can be synchronized from scratch
         (see https://metaapi.cloud/docs/client/websocket/api/removeHistory/).
 
+        Args:
+            application: Application to remove history for.
+
         Returns:
             A coroutine resolving when the history is cleared.
         """
         self._historyStorage.reset()
-        return self._websocketClient.remove_history(self._account.id)
+        return self._websocketClient.remove_history(self._account.id, application)
 
     def remove_application(self):
         """Clears the order and transaction history of a specified application and removes application (see
@@ -228,7 +238,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_BUY', 'symbol': symbol, 'volume': volume}
         if stop_loss:
@@ -254,7 +264,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_SELL', 'symbol': symbol, 'volume': volume}
         if stop_loss:
@@ -281,7 +291,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_BUY_LIMIT', 'symbol': symbol, 'volume': volume,
                         'openPrice': open_price}
@@ -309,7 +319,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_SELL_LIMIT', 'symbol': symbol, 'volume': volume,
                         'openPrice': open_price}
@@ -337,7 +347,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_BUY_STOP', 'symbol': symbol, 'volume': volume,
                         'openPrice': open_price}
@@ -365,7 +375,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_TYPE_SELL_STOP', 'symbol': symbol, 'volume': volume,
                         'openPrice': open_price}
@@ -376,6 +386,64 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
+    def create_stop_limit_buy_order(self, symbol: str, volume: float, open_price: float, stop_limit_price: float,
+                                    stop_loss: float = None, take_profit: float = None,
+                                    options: PendingTradeOptions = None):
+        """Creates a stop limit buy order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
+
+        Args:
+            symbol: Symbol to trade.
+            volume: Order volume.
+            open_price: Order limit price.
+            stop_limit_price: The limit order price for the stop limit order.
+            stop_loss: Optional stop loss price.
+            take_profit: Optional take profit price.
+            options: Optional trade options.
+
+        Returns:
+            A coroutine resolving with trade result.
+
+        Raises:
+            TradeException: On trade error, check error properties for error code details.
+        """
+        trade_params = {'actionType': 'ORDER_TYPE_BUY_STOP_LIMIT', 'symbol': symbol, 'volume': volume,
+                        'openPrice': open_price, 'stopLimitPrice': stop_limit_price}
+        if stop_loss:
+            trade_params['stopLoss'] = stop_loss
+        if take_profit:
+            trade_params['takeProfit'] = take_profit
+        trade_params.update(options or {})
+        return self._websocketClient.trade(self._account.id, trade_params)
+
+    def create_stop_limit_sell_order(self, symbol: str, volume: float, open_price: float, stop_limit_price: float,
+                                     stop_loss: float = None, take_profit: float = None,
+                                     options: PendingTradeOptions = None):
+        """Creates a stop limit sell order (see https://metaapi.cloud/docs/client/websocket/api/trade/).
+
+        Args:
+            symbol: Symbol to trade.
+            volume: Order volume.
+            open_price: Order limit price.
+            stop_limit_price: The limit order price for the stop limit order.
+            stop_loss: Optional stop loss price.
+            take_profit: Optional take profit price.
+            options: Optional trade options.
+
+        Returns:
+            A coroutine resolving with trade result.
+
+        Raises:
+            TradeException: On trade error, check error properties for error code details.
+        """
+        trade_params = {'actionType': 'ORDER_TYPE_SELL_STOP_LIMIT', 'symbol': symbol, 'volume': volume,
+                        'openPrice': open_price, 'stopLimitPrice': stop_limit_price}
+        if stop_loss:
+            trade_params['stopLoss'] = stop_loss
+        if take_profit:
+            trade_params['takeProfit'] = take_profit
+        trade_params.update(options or {})
+        return self._websocketClient.trade(self._account.id, trade_params)
+
     def modify_position(self, position_id: str, stop_loss: float = None, take_profit: float = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Modifies a position (see https://metaapi.cloud/docs/client/websocket/api/trade/).
@@ -411,7 +479,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'POSITION_PARTIAL', 'positionId': position_id, 'volume': volume}
         trade_params.update(options or {})
@@ -429,12 +497,32 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'POSITION_CLOSE_ID', 'positionId': position_id}
         trade_params.update(options or {})
         return self._websocketClient.trade(self._account.id, trade_params)
 
+    def close_by(self, position_id: str, opposite_position_id: str, options: MarketTradeOptions = None) -> \
+            'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
+        """Fully closes a position (see https://metaapi.cloud/docs/client/websocket/api/trade/).
+
+        Args:
+            position_id: Position id to close by opposite position.
+            opposite_position_id: Opposite position id to close.
+            options: Optional trade options.
+
+        Returns:
+            A coroutine resolving with trade result.
+
+        Raises:
+            TradeException: On trade error, check error properties for error code details.
+        """
+        trade_params = {'actionType': 'POSITION_CLOSE_BY', 'positionId': position_id,
+                        'closeByPositionId': opposite_position_id}
+        trade_params.update(options or {})
+        return self._websocketClient.trade(self._account.id, trade_params)
+
     def close_positions_by_symbol(self, symbol: str, options: MarketTradeOptions = None) -> \
             'Coroutine[asyncio.Future[MetatraderTradeResponse]]':
         """Closes positions by a symbol (see https://metaapi.cloud/docs/client/websocket/api/trade/).
@@ -447,7 +535,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'POSITIONS_CLOSE_SYMBOL', 'symbol': symbol}
         trade_params.update(options or {})
@@ -468,7 +556,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         trade_params = {'actionType': 'ORDER_MODIFY', 'orderId': order_id, 'openPrice': open_price}
         if stop_loss:
@@ -488,7 +576,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         return self._websocketClient.trade(self._account.id, {'actionType': 'ORDER_CANCEL', 'orderId': order_id})
 
@@ -530,7 +618,11 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine which resolves when subscription is initiated.
         """
-        return self._websocketClient.subscribe(self._account.id)
+        try:
+            return await self._websocketClient.subscribe(self._account.id)
+        except Exception as err:
+            if err.__class__.__name__ != 'TimeoutException':
+                raise err
 
     def subscribe_to_market_data(self, symbol: str) -> Coroutine:
         """Subscribes on market data of specified symbol (see
@@ -542,8 +634,18 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             Promise which resolves when subscription request was processed.
         """
+        self._subscriptions[symbol] = True
         return self._websocketClient.subscribe_to_market_data(self._account.id, symbol)
 
+    @property
+    def subscribed_symbols(self) -> List[str]:
+        """Returns list of the symbols connection is subscribed to.
+
+        Returns:
+            List of the symbols connection is subscribed to.
+        """
+        return list(self._subscriptions.keys())
+
     def get_symbol_specification(self, symbol: str) -> 'Coroutine[asyncio.Future[MetatraderSymbolSpecification]]':
         """Retrieves specification for a symbol (see
         https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/getSymbolSpecification/).
@@ -568,6 +670,17 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         return self._websocketClient.get_symbol_price(self._account.id, symbol)
 
+    def save_uptime(self, uptime: Dict):
+        """Sends client uptime stats to the server.
+
+        Args:
+            uptime: Uptime statistics to send to the server.
+
+        Returns:
+            A coroutine which resolves when uptime statistics is submitted.
+        """
+        return self._websocketClient.save_uptime(self._account.id, uptime)
+
     @property
     def terminal_state(self) -> TerminalState:
         """Returns local copy of terminal state.
@@ -608,16 +721,18 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine which resolves when the asynchronous event is processed.
         """
-        try:
-            await self.synchronize()
-        except Exception as err:
-            print(f'[{datetime.now().isoformat()}] MetaApi websocket client for account {self._account.id} '
-                  'failed to synchronize', err)
+        key = random_id(32)
+        self._shouldSynchronize = key
+        self._synchronizationRetryIntervalInSeconds = 1
+        self._synchronized = False
+        await self._ensure_synchronized(key)
 
     async def on_disconnected(self):
         """Invoked when connection to MetaTrader terminal terminated"""
         self._lastDisconnectedSynchronizationId = self._lastSynchronizationId
         self._lastSynchronizationId = None
+        self._shouldSynchronize = None
+        self._synchronized = False
 
     async def on_deal_synchronization_finished(self, synchronization_id: str):
         """Invoked when a synchronization of history deals on a MetaTrader account have finished.
@@ -677,6 +792,8 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         synchronization_id = opts['synchronizationId'] if 'synchronizationId' in opts else None
         timeout_in_seconds = opts['timeoutInSeconds'] if 'timeoutInSeconds' in opts else 300
         interval_in_milliseconds = opts['intervalInMilliseconds'] if 'intervalInMilliseconds' in opts else 1000
+        application_pattern = opts['applicationPattern'] if 'applicationPattern' in opts \
+            else ('CopyFactory.*|RPC' if self._account.application == 'CopyFactory' else 'RPC')
         synchronized = await self.is_synchronized(synchronization_id)
         while not synchronized and (start_time + timedelta(seconds=timeout_in_seconds) > datetime.now()):
             await asyncio.sleep(interval_in_milliseconds / 1000)
@@ -688,14 +805,61 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
                                                                                  self._lastDisconnectedSynchronizationId
                                                                                  or 'None'))
         time_left_in_seconds = max(0, timeout_in_seconds - (datetime.now() - start_time).total_seconds())
-        await self._websocketClient.wait_synchronized(self._account.id, opts['applicationPattern'] if
-                                                      'applicationPattern' in opts else '.*', time_left_in_seconds)
+        await self._websocketClient.wait_synchronized(self._account.id, application_pattern, time_left_in_seconds)
 
-    def close(self):
+    async def close(self):
         """Closes the connection. The instance of the class should no longer be used after this method is invoked."""
         if not self._closed:
+            self._shouldSynchronize = None
+            await self._websocketClient.unsubscribe(self._account.id)
             self._websocketClient.remove_synchronization_listener(self._account.id, self)
             self._websocketClient.remove_synchronization_listener(self._account.id, self._terminalState)
             self._websocketClient.remove_synchronization_listener(self._account.id, self._historyStorage)
+            self._websocketClient.remove_synchronization_listener(self._account.id, self._healthMonitor)
             self._connection_registry.remove(self._account.id)
+            self._healthMonitor.stop()
             self._closed = True
+
+    @property
+    def synchronized(self) -> bool:
+        """Returns synchronization status.
+
+        Returns:
+            Synchronization status.
+        """
+        return self._synchronized
+
+    @property
+    def account(self) -> MetatraderAccountModel:
+        """Returns MetaApi account.
+
+        Returns:
+            MetaApi account.
+        """
+        return self._account
+
+    @property
+    def health_monitor(self) -> ConnectionHealthMonitor:
+        """Returns connection health monitor instance.
+
+        Returns:
+            Connection health monitor instance.
+        """
+        return self._healthMonitor
+
+    async def _ensure_synchronized(self, key):
+        try:
+            await self.synchronize()
+            for symbol in self.subscribed_symbols:
+                await self.subscribe_to_market_data(symbol)
+            self._synchronized = True
+            self._synchronizationRetryIntervalInSeconds = 1
+        except Exception as err:
+            print(f'[{datetime.now().isoformat()}] MetaApi websocket client for account ' + self.account.id +
+                  ' failed to synchronize', err)
+            if self._shouldSynchronize == key:
+                def restart_ensure_sync():
+                    asyncio.create_task(self._ensure_synchronized(key))
+                self._ensure_sync_timer = Timer(self._synchronizationRetryIntervalInSeconds, restart_ensure_sync)
+                self._ensure_sync_timer.start()
+                self._synchronizationRetryIntervalInSeconds = min(self._synchronizationRetryIntervalInSeconds * 2, 300)
