@@ -109,7 +109,7 @@ class AutoMockAccount(MetatraderAccount):
 account = MockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
 auto_account = AutoMockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
 client = MockClient('token')
-api = MetaApiConnection(client, account, MagicMock(), MagicMock())
+api: MetaApiConnection = None
 
 
 @pytest.fixture(autouse=True)
@@ -407,8 +407,8 @@ class TestMetaApiConnection:
     async def test_remove_history(self):
         """Should remove history."""
         client.remove_history = AsyncMock()
-        await api.remove_history()
-        client.remove_history.assert_called_with('accountId')
+        await api.remove_history('app')
+        client.remove_history.assert_called_with('accountId', 'app')
 
     @pytest.mark.asyncio
     async def test_create_market_buy_order(self):
@@ -510,6 +510,40 @@ class TestMetaApiConnection:
                                                       'takeProfit': 2.0, 'comment': 'comment',
                                                       'clientId': 'TE_GBPUSD_7hyINWqAlE'})
 
+    @pytest.mark.asyncio
+    async def test_create_stop_limit_buy_order(self):
+        """Should create stop limit buy order."""
+        trade_result = {
+            'error': 10009,
+            'description': 'TRADE_RETCODE_DONE',
+            'orderId': 46870472
+        }
+        client.trade = AsyncMock(return_value=trade_result)
+        actual = await api.create_stop_limit_buy_order('GBPUSD', 0.07, 1.5, 1.4, 0.9, 2.0, {
+            'comment': 'comment', 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+        assert actual == trade_result
+        client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_BUY_STOP_LIMIT', 'symbol': 'GBPUSD',
+                                                      'volume': 0.07, 'openPrice': 1.5, 'stopLimitPrice': 1.4,
+                                                      'stopLoss': 0.9, 'takeProfit': 2.0, 'comment': 'comment',
+                                                      'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+
+    @pytest.mark.asyncio
+    async def test_create_stop_limit_sell_order(self):
+        """Should create stop limit sell order."""
+        trade_result = {
+            'error': 10009,
+            'description': 'TRADE_RETCODE_DONE',
+            'orderId': 46870472
+        }
+        client.trade = AsyncMock(return_value=trade_result)
+        actual = await api.create_stop_limit_sell_order('GBPUSD', 0.07, 1.0, 1.1, 2.0, 0.9, {
+            'comment': 'comment', 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+        assert actual == trade_result
+        client.trade.assert_called_with('accountId', {'actionType': 'ORDER_TYPE_SELL_STOP_LIMIT', 'symbol': 'GBPUSD',
+                                                      'volume': 0.07, 'openPrice': 1.0, 'stopLimitPrice': 1.1,
+                                                      'stopLoss': 2.0, 'takeProfit': 0.9, 'comment': 'comment',
+                                                      'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+
     @pytest.mark.asyncio
     async def test_modify_position(self):
         """Should modify position."""
@@ -551,6 +585,22 @@ class TestMetaApiConnection:
         assert actual == trade_result
         client.trade.assert_called_with('accountId', {'actionType': 'POSITION_CLOSE_ID', 'positionId': '46870472'})
 
+    @pytest.mark.asyncio
+    async def test_close_position_by_opposite(self):
+        """Should close position by an opposite one."""
+        trade_result = {
+            'error': 10009,
+            'description': 'TRADE_RETCODE_DONE',
+            'positionId': '46870472',
+            'closeByPositionId': '46870482'
+        }
+        client.trade = AsyncMock(return_value=trade_result)
+        actual = await api.close_by('46870472', '46870482', {'comment': 'comment', 'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+        assert actual == trade_result
+        client.trade.assert_called_with('accountId', {'actionType': 'POSITION_CLOSE_BY', 'positionId': '46870472',
+                                                      'closeByPositionId': '46870482', 'comment': 'comment',
+                                                      'clientId': 'TE_GBPUSD_7hyINWqAlE'})
+
     @pytest.mark.asyncio
     async def test_close_positions_by_symbol(self):
         """Should close positions by symbol."""
@@ -667,6 +717,13 @@ class TestMetaApiConnection:
 
         client.get_symbol_price.assert_called_with('accountId', 'AUDNZD')
 
+    @pytest.mark.asyncio
+    async def test_save_uptime_stats(self):
+        """Should save uptime stats to the server."""
+        client.save_uptime = AsyncMock()
+        await api.save_uptime({'1h': 100})
+        client.save_uptime.assert_called_with('accountId', {'1h': 100})
+
     @pytest.mark.asyncio
     async def test_initialize(self):
         """Should initialize listeners, terminal state and history storage for accounts with user sync mode."""
@@ -708,13 +765,42 @@ class TestMetaApiConnection:
             client.synchronize.assert_called_with('accountId', 'synchronizationId', date('2020-01-01T00:00:00.000Z'),
                                                   date('2020-01-02T00:00:00.000Z'))
 
+    @pytest.mark.asyncio
+    async def test_maintain_sync(self):
+        """Should maintain synchronization if connection has failed."""
+        with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
+            client.synchronize = AsyncMock(side_effect=[Exception('test error'), None])
+            api = MetaApiConnection(client, account, None, MagicMock())
+            await api.history_storage.on_history_order_added({'doneTime': date('2020-01-01T00:00:00.000Z')})
+            await api.history_storage.on_deal_added({'time': date('2020-01-02T00:00:00.000Z')})
+            await api.on_connected()
+            client.synchronize.assert_called_with('accountId', 'synchronizationId', date('2020-01-01T00:00:00.000Z'),
+                                                  date('2020-01-02T00:00:00.000Z'))
+
+    @pytest.mark.asyncio
+    async def test_restore_market_data_subs_on_sync(self):
+        """Should restore market data subscriptions on synchronization."""
+        with patch('lib.metaApi.metaApiConnection.random_id', return_value='synchronizationId'):
+            client.synchronize = AsyncMock()
+            client.subscribe_to_market_data = AsyncMock()
+            api = MetaApiConnection(client, account, None, MagicMock())
+            await api.history_storage.on_history_order_added({'doneTime': date('2020-01-01T00:00:00.000Z')})
+            await api.history_storage.on_deal_added({'time': date('2020-01-02T00:00:00.000Z')})
+            await api.subscribe_to_market_data('EURUSD')
+            await api.on_connected()
+            client.synchronize.assert_called_with('accountId', 'synchronizationId', date('2020-01-01T00:00:00.000Z'),
+                                                  date('2020-01-02T00:00:00.000Z'))
+            assert client.subscribe_to_market_data.call_count == 2
+
     @pytest.mark.asyncio
     async def test_unsubscribe_from_events_on_close(self):
         """Should unsubscribe from events on close."""
         client.add_synchronization_listener = MagicMock()
         client.remove_synchronization_listener = MagicMock()
+        client.unsubscribe = AsyncMock()
         api = MetaApiConnection(client, account, MagicMock(), MagicMock())
-        api.close()
+        await api.close()
+        client.unsubscribe.assert_any_call('accountId')
         client.remove_synchronization_listener.assert_any_call('accountId', api)
         client.remove_synchronization_listener.assert_any_call('accountId', api.terminal_state)
         client.remove_synchronization_listener.assert_any_call('accountId', api.history_storage)
